<?php 

require FCSFW_PLUGIN_DIR_PATH . 'vendor/autoload.php';

use UnitConverter\UnitConverter;


    /**
     * Function that get the chosen shipping method details for a cart item
     */
    function fcsfw_get_cart_item_shipping_method( $cart_item_key ){
        $chosen_shippings = WC()->session->get( 'chosen_shipping_methods' ); // The chosen shipping methods
        //$logger->debug('$chosen_shippings', array('source' => 'FlavorCloudIT'));
        foreach( WC()->cart->get_shipping_packages() as $id => $package ) {
            $chosen = $chosen_shippings[$id]; // The chosen shipping method
            if( isset($package['contents'][$cart_item_key]) && WC()->session->__isset('shipping_for_package_'.$id) ) {
                return WC()->session->get('shipping_for_package_'.$id)['rates'][$chosen];
            }
        }
    }

    /**
     * Function that returns the weight and size units
     */
    function fcsfw_get_woocommerce_weight_and_dimension_units() {
        $weight_unit = get_option( 'woocommerce_weight_unit');
        if ($weight_unit == 'lbs') {
            $weight_unit = 'lb';
        }
        $dimension_unit = get_option( 'woocommerce_dimension_unit' );
    
        // Convert the weight unit to KG or LB
        switch($weight_unit) {
            case 'kg':
            case 'g':
                $weight_unit = 'KG';
                break;
            case 'lbs':
            case 'oz':
            default:
                $weight_unit = 'LB';
                break;
        }
    
        // Convert the dimension unit to CM or IN
        switch($dimension_unit) {
            case 'cm':
            case 'mm':
            case 'm':
                $dimension_unit = 'CM';
                break;
            case 'in':
            case 'yd':
            default:
                $dimension_unit = 'IN';
                break;
        }
    
        return array('weight_unit' => $weight_unit, 'dimension_unit' => $dimension_unit);
    }

    /**
     * Helper Function to convert units
     */
    function fcsfw_convertUnits($value, $from, $to) {
        if (empty($value) || empty($from) || empty($to)) {
            return "Invalid input: one or more of the parameters is empty.";
        }
        $converter = UnitConverter::createBuilder()
        ->addSimpleCalculator()
        ->addDefaultRegistry()
        ->build();
    
        return (float)$converter->convert($value)->from($from)->to($to);
    }
    





add_action('woocommerce_checkout_create_order', 'add_custom_meta_field_for_order', 10, 2);
function add_custom_meta_field_for_order($order, $data) {
    if (!isset($data['shipping_method']) || !is_array($data['shipping_method'])) {
        return;
    }

    // Determine if the order contains a subscription product
    $contains_subscription = false;
    foreach ($order->get_items() as $item) {
        $product = $item->get_product();
        if ($product && method_exists($product, 'is_type')) {
            if ($product->is_type('subscription')) {
                $contains_subscription = true;
                break;
            }
        }
    }

    // For Simple and Variable Products
    if (!$contains_subscription && count($data['shipping_method']) == 1) {
        $order->update_meta_data('flavorcloud_shipping_rate_id', $data['shipping_method'][0]);
    }
    // For Subscription Products
    elseif ($contains_subscription) {
        $shipping_methods = array_values($data['shipping_method']); // Reset keys
        $order->update_meta_data('flavorcloud_shipping_rate_id', $shipping_methods[0]); // For the first shipping method
        if (isset($shipping_methods[1])) {
            $order->update_meta_data('flavorcloud_shipping_rate_id_subscription', $shipping_methods[1]); // For the second shipping method (recurring)
        }
    }

}



    
    // Disable reocurring shipping method DELETE
    //add_filter( 'woocommerce_cart_calculate_fees', 'add_recurring_postage_fees', 10, 1 );
    function add_recurring_postage_fees( $cart ) {
        if ( ! empty( $cart->recurring_cart_key ) ) {
            remove_action( 'woocommerce_cart_totals_after_order_total', array( 'WC_Subscriptions_Cart', 'display_recurring_totals' ), 10 );
            remove_action( 'woocommerce_review_order_after_order_total', array( 'WC_Subscriptions_Cart', 'display_recurring_totals' ), 10 );
        }
    }
    

    /**
     * Save shipping method details in order line items (product) as custom order item meta data
     * This meta data is used to generate a shipping label 
     */
    //add_action( 'woocommerce_checkout_create_order_line_item', 'fcsfw_woocommerce_checkout_create_order_line_item', 10, 4 );
    function fcsfw_woocommerce_checkout_create_order_line_item( $item, $cart_item_key, $values, $order ) {

        //$logger = wc_get_logger();
        // Load shipping rate for this item
        $rate = fcsfw_get_cart_item_shipping_method( $cart_item_key );
        //$logger->debug(print_r( $rate, true ), array('source' => 'TEST'));
        //$logger->debug(print_r( $cart_item_key, true ), array('source' => 'TEST'));
        //$logger->debug(print_r(  $values, true ), array('source' => 'TEST'));
        //error_log( print_r( $rate->method_id, true ) );
        //fcsfw_logger_sp($rate->method_id);
        //die();
        // flavor-cloud
        //var_dump($rate);
        //die();
        if ($rate->method_id == 'flavor-cloud'):
        // Set custom order item meta data
        $order->update_meta_data( 'flavorcloud_shipping_rate_id', $rate->id ); // the shipping rate ID
       // $logger->debug('Meta Saved:flavorcloud_shipping_rate_id', array('source' => 'TEST'));
        endif;
    }
    

    /**
	 * Clear FlavorCloud transients.
	 */
	function fcsfw_clear_transients() {
		global $wpdb;
		$wpdb->query( "DELETE FROM `$wpdb->options` WHERE `option_name` LIKE ('_transient_flavorcloud_quote_%') OR `option_name` LIKE ('_transient_timeout_flavorcloud_quote_%')" );
	}


     /**
	 * Clear Shipping Cache 
     * Additional tax/fees do not work in some cases if this is not disabled.
     */
    add_action( 'wp_loaded', 'before_shipping');
    function before_shipping( $checkout ) {
        if (!is_admin() && defined( 'DOING_AJAX' ) && DOING_AJAX ) {

        // check the user credentials here and if the condition is met set the shipping method
        $packages = WC()->cart->get_shipping_packages();
        foreach ($packages as $key => $value) {
            $shipping_session = "shipping_for_package_$key";
            unset(WC()->session->$shipping_session);
        }
    }

    }
