<?php
/**
 * FlavorCloud Shipping For Woocommerce Admin
 */


/**
 * Adding HS Code Support to woocomerce Simple + Variable Product Support. 
 */
add_action('woocommerce_product_options_sku','fcsfw_add_admin_fields' );
function fcsfw_add_admin_fields(){
    global $woocommerce;
    $_product = wc_get_product( get_the_ID() );
    if( $_product->is_type( 'simple' ) ) :

        $countries_obj         = new WC_Countries();
        $origincountrycode     = $countries_obj->__get('countries');
        $origincountrycode     = [0 =>__( 'Select a Country', 'woocommerce')] + $origincountrycode;
        // Current Value  
        $origincountrycode_selected = get_post_meta(get_the_ID(),"_origincountrycode", true );
        echo '<div class="options_group"><p><span style="display: block;font-size: 14px;padding: 6px 0px;font-weight: bold;">Customs Information</span> Used by border officers to calculate duties when shipping internationally. Shown on customs forms you print during fulfilment.</p>';
        
        // Exclude from FlavorCloud Checkbox
        woocommerce_wp_checkbox( array(
            'id'            => '_exclude_flavorcloud',
            'label'         => __( 'Exclude This Product', 'woocommerce' ),
            'desc_tip'    => 'true',
            'description'   => __( 'Checking this box will exclude this product from FlavorCloud, You can use this option for addon products, gifts or Packing material.', 'woocommerce' ),
        ) );

        woocommerce_wp_select( array(
            'id'      => '_origincountrycode',
            'label'   => __( 'Country of origin', 'woocommerce' ),
            'options' =>  $origincountrycode,
            'value'   => $origincountrycode_selected,
            'desc_tip'    => 'true',
            'description' => __('Please select where the product/item is manufactured.','woocommerce')
        ) );
        woocommerce_wp_text_input( array(
            'id'          => '_hscode',
            'label'       => __('HS Code','woocommerce'),
            'placeholder' => 'Harmonized System Code',
            'desc_tip'    => 'true',
            'description' => __('HS code is used by Customs authorities, statistical agencies, and other government regulatory bodies.','woocommerce')
        ) );
        woocommerce_wp_text_input( array(
            'id'          => '_material',
            'label'       => __('Material','woocommerce'),
            'placeholder' => 'Product Material ex: Wood / Cotton / Glass / Plastic',
            'desc_tip'    => 'true',
            'description' => __('Here you can add the material your product is made of, for food items you can add the list of ingredients.','woocommerce')
        ) );
        echo '</div>';
    endif;
}
add_action( 'woocommerce_process_product_meta', 'save_fcsfw_admin_fields', 10, 1 );
function save_fcsfw_admin_fields( $post_id ){
    $_product = wc_get_product( get_the_ID() );
    if( $_product->is_type( 'simple' ) ) :
        if( isset($_POST['_origincountrycode']) )
            update_post_meta( $post_id, '_origincountrycode', esc_attr( $_POST['_origincountrycode'] ) );
        if( isset($_POST['_hscode']) )
            update_post_meta( $post_id, '_hscode', esc_attr( $_POST['_hscode'] ) );
        if( isset($_POST['_material']) )
            update_post_meta( $post_id, '_material', esc_attr( $_POST['_material'] ) );
        // Save '_exclude_flavorcloud' field
        $exclude_flavorcloud = isset($_POST['_exclude_flavorcloud']) ? 'yes' : 'no';
        update_post_meta( $post_id, '_exclude_flavorcloud', $exclude_flavorcloud );
    endif;
}
add_action( 'admin_footer', 'custom_admin_js' );
function custom_admin_js() {
    global $post, $pagenow;

    // Ensure we're on a product edit page
    if ( $pagenow != 'post.php' || $post->post_type != 'product' ) {
        return;
    }

    ?>
<script>
jQuery(document).ready(function($) {
    function handleFlavorCloudFields() {
        // Handles both simple products and variations
        $('.options_group, .woocommerce_variation').each(function() {
            var checkbox = $(this).find('input[name^="_exclude_flavorcloud"]');
            var fields = $(this).find('input[name^="_origincountrycode"], input[name^="_hscode"], input[name^="_material"], select[name^="_origincountrycode"]');
            if(checkbox.is(':checked')){
                fields.prop('disabled', true);
            } else {
                fields.prop('disabled', false);
            }
        });
    }

    // On page load
    handleFlavorCloudFields();

    // On checkbox change
    $(document).on('change', 'input[name^="_exclude_flavorcloud"]', function(){
        handleFlavorCloudFields();
    });

    // Since variations are loaded dynamically, we also need to reapply the logic when a new variation is loaded
    $(document).on('woocommerce_variations_loaded', handleFlavorCloudFields);
});
</script>
<style>
.woocommerce_variable_attributes label[for^="_exclude_flavorcloud"] {
    margin-left:4px;
}
</style>
    <?php
}
// Variable Products
add_action( 'woocommerce_variation_options_pricing', 'add_fcsfw_admin_variation_fields', 10, 3 );
function add_fcsfw_admin_variation_fields( $loop, $variation_data, $variation ){
    $countries_obj         = new WC_Countries();
    $origincountrycode     = $countries_obj->__get('countries');
    $origincountrycode     = [0 =>__( 'Select a Country', 'woocommerce')] + $origincountrycode;
    $variation_origincountrycode = get_post_meta($variation->ID,"_origincountrycode", true );
    $variation_hscode = get_post_meta($variation->ID,"_hscode", true );
    $variation_material = get_post_meta($variation->ID,"_material", true );
    $exclude_flavorcloud = get_post_meta($variation->ID,"_exclude_flavorcloud", true ) == 'yes';
    if( ! $variation_origincountrycode ) $variation_origincountrycode = 0;
    if( ! $variation_hscode ) $variation_hscode = "";
    if( ! $variation_material ) $variation_material = "";
    woocommerce_wp_checkbox(
        array(
            'id'          => '_exclude_flavorcloud[' . $loop . ']',
            'label'       => __( 'Exclude This Variation', 'woocommerce' ),
            'desc_tip'    => 'true',
            'description' => __( 'Checking this box will exclude this product variation from FlavorCloud, You can use this option for addon products, gifts or Packing material.', 'woocommerce' ),
            'value'       => $exclude_flavorcloud ? 'yes' : 'no',
        )
    );
    echo '<p><span style="display: block;font-size: 14px;padding: 6px 0px;font-weight: bold;">Customs Information</span> Used by border officers to calculate duties when shipping internationally. Shown on customs forms you print during fulfilment.</p>';
    woocommerce_wp_select( array(
        'id'      => '_origincountrycode['.$loop.']',
        'label'   => __( 'Country of origin', 'woocommerce' ),
        'options' =>  $origincountrycode,
        'value'       => get_post_meta($variation->ID,"_origincountrycode", true ),
        'desc_tip'    => 'true',
        'description' => __('Please select where the product/item is manufactured.','woocommerce'),
        'class' => 'form-row-full',
    ) );
    woocommerce_wp_text_input( array(
        'id'          => '_hscode['.$loop.']',
        'label'       => __('HSCode','woocommerce'),
        'placeholder' => 'Harmonized System Code',
        'class' => 'short wc_input_price',
        'desc_tip'    => 'true',
        'description' => __('HS code are used by Customs authorities, statistical agencies, and other government regulatory bodies.','woocommerce'),
        'value'       => get_post_meta($variation->ID,"_hscode", true ),
    ) );
    woocommerce_wp_text_input( array(
        'id'          => '_material['.$loop.']',
        'label'       => __('Material','woocommerce'),
        'placeholder' => 'Product Material ex: Wood',
        'class' => 'short',
        'desc_tip'    => 'true',
        'description' => __('Here you can add the material your product is made of, for food items you can add the list of ingredients.  ','woocommerce'),
        'value'       => get_post_meta($variation->ID,"_material", true ),
    ) );
}
// Save Variation Barcode custom field value
add_action( 'woocommerce_save_product_variation', 'save_fcsfw_admin_variation_fields', 10, 2 );
function save_fcsfw_admin_variation_fields( $variation_id, $i ){
    if( isset($_POST['_origincountrycode'][$i]) )
    update_post_meta( $variation_id, '_origincountrycode', sanitize_text_field($_POST['_origincountrycode'][$i]) );
    if( isset($_POST['_hscode'][$i]) )
        update_post_meta( $variation_id, '_hscode', sanitize_text_field($_POST['_hscode'][$i]) );
    if( isset($_POST['_material'][$i]) )
        update_post_meta( $variation_id, '_material', sanitize_text_field($_POST['_material'][$i]) );
    // Save the checkbox
    $exclude_flavorcloud = isset($_POST['_exclude_flavorcloud'][$i]) ? 'yes' : 'no';
    update_post_meta( $variation_id, '_exclude_flavorcloud', $exclude_flavorcloud );
}



/**
 * Enquire Admin JS
 */
function wp_fcsfw_scripts() {
    if (get_post_type() === 'shop_order' ) :
    wp_enqueue_script('wp_fcsfw_scripts', FCSFW_PLUGIN_DIR_URL . 'includes/js/fcsfw-main.js', '', '1.1.0', true);
    wp_localize_script( 'wp_fcsfw_scripts', 'wp_fcsfw_ajax', array(
      'ajax_url' => admin_url( 'admin-ajax.php' ),
      'nonce' => wp_create_nonce( 'wp-fcsfw-nonce' ),
      'order_id' => get_the_ID()
    ) );
    endif; 
}
add_action('admin_enqueue_scripts', 'wp_fcsfw_scripts');

add_action('wp_ajax_wp_fcsfw_ajax', 'ajax_event_form');
function ajax_event_form(){
//sleep(5);
   var_dump(FlavorCloud_API_ShipmentRequest($_POST['order_id'],$_POST['fcsfw_pickup_date']));
   die;
}


/**
 * Add Metabox on edit order page. 
 */
add_action( 'add_meta_boxes', 'fcsfw_add_order_meta_box' );
    function fcsfw_add_order_meta_box()
    {
        $post_id = get_the_ID();   
        if( get_post_type(  $post_id ) === 'shop_order' ) :
            //global $post;
            $post_id = get_the_ID();
            $order = wc_get_order( $post_id );
            //var_dump($post_id);
            //die();
            if ($order->get_shipping_country() != "US"):
            add_meta_box( 'flavor_cloud_ship', __('Flavor Cloud','woocommerce'), 'fcsfw_add_order_packaging', 'shop_order', 'side', 'core' );
            endif;
        endif;
    }
// Adding Meta field in the meta container admin shop_order pages
    function fcsfw_add_order_packaging()
    {

        //set_option( '','_cloud_flare_shipmentrequest_response', '');
        //global $post;
        $post_id = get_the_ID();

        // Check If this order has been shipped 
        $ShipmentRequest_Response = json_decode(get_post_meta( $post_id ,'_Cloud_Flare_ShipmentRequest_Response', true));
        // Check If this order has been shipped 
        $ShipmentRequest_Box = json_decode(get_post_meta( $post_id ,'_Cloud_Flare_ShipmentRequest_Box', true));     
        echo '<div id="fcsfw_meta">';
        if( empty( get_post_meta( $post_id, 'flavorcloud_shipping_rate_id', true ) ) ):
            //echo '<h4> Th</span> </h4>'; 
            echo '<p>This order cannot be shipped with the FlavorCloud API.</p>';
            echo '</div>';
            return;
        endif;

        if (isset($ShipmentRequest_Response->CustomsInvoiceURL) && is_object($ShipmentRequest_Response) && wp_http_validate_url($ShipmentRequest_Response->CustomsInvoiceURL)):
         echo '<h4> Shippement status: <span style="color:#1d9a1d;">Label Created</span> </h4>'; 
         //update_post_meta( '7319', '_Cloud_Flare_ShipmentRequest_Box', '{"Name":"Priority Mail Box 4","Length":7.25,"Width":7.25,"Height":6.5,"Weight":0.3,"MaxWeight":20}' );
         echo '<b>Shipment Box:</b> '.$ShipmentRequest_Box->Name;
         echo '<br>';
         echo '<small>Length:'.$ShipmentRequest_Box->Length.' Width:'.$ShipmentRequest_Box->Width.' Height:'.$ShipmentRequest_Box->Height.'</small>';
         echo '<br>';
         echo '<br>';
         //  0 => '{"Name":"Priority Mail Box 4","Length":7.25,"Width":7.25,"Height":6.5,"Weight":0.3,"MaxWeight":20}',
         echo '<b>ShipmentID:</b> '.$ShipmentRequest_Response->ShipmentID;
         echo '<br>';
         echo '<b>TrackingNumber:</b> '.$ShipmentRequest_Response->TrackingNumber;
         echo '<br>';
         echo '<b>Carrier:</b> '.$ShipmentRequest_Response->Carrier;
         echo '<br>';
         echo '<a target="_blank" href="https://app.flavorcloud.com/brandedTracking?ref='.$ShipmentRequest_Response->Reference.'&tracking_number='.$ShipmentRequest_Response->TrackingNumber.'&carrier='.$ShipmentRequest_Response->Carrier.'&destination=">Track Package</a>';
         echo '<br>';
         echo '<br>';
         foreach ($ShipmentRequest_Response->LabelUrl as $key => $LabelUrl) {
            echo '<a href="'.$LabelUrl.'" target="_blank" class="button button-primary"  value="">Download Label</a>';
         }
         echo '<br>';
         echo '<br>';
         echo '<a href="'.$ShipmentRequest_Response->CustomsInvoiceURL.'" target="_blank" class="button button-primary"  value="">Download Customs Invoice</a>';

        else:
         echo '<h4> Status: Not Shipped </h4>'; 
         echo '<p>A shipping label and Customs invoice will be generated when you click the Create Shipment button. </p>';
            $meta_field_data = get_post_meta( $post_id, '_fcsfw_pickup_date', true ) ? get_post_meta( $post_id, '_fcsfw_pickup_date', true ) : '';
            $flavorcloud_shipping_rate_id = get_post_meta($post_id,"flavorcloud_shipping_rate_id", true );
            // echo $flavorcloud_shipping_rate_id;
            // FlavorCloud_API_ShipmentRequest();
            
            
            echo '<input type="hidden" id="fcsfw_order_meta_field_nonce" name="fcsfw_order_meta_field_nonce" value="' . wp_create_nonce() . '">
            <p style="border-bottom:solid 1px #eee;padding-bottom:13px;">
                <label for="my_field_name">Shipment Pickup Date:</label>';
                
                echo '<input type="text" id="fcsfw_pickup_date" class="fcsfw_pickup_date" style="width:250px;";" name="fcsfw_pickup_date" placeholder="' . $meta_field_data . '" value="' . $meta_field_data . '"></p>';
                echo '<a id="fcsfw-ship-order" class="button button-primary"  value="">Create Shipment</a>';
                echo '<br>';
                echo '<small style="margin-top:10px;display: block;">Flavor Cloud will only charge your account once the package is scanned by the carrier.</small>';

            // Datepicker
            
                    // The displayed value using GET method
        if ( isset( $_GET['fcsfw_order_meta_field_nonce'] ) && ! empty( $_GET['fcsfw_order_meta_field_nonce'] ) ) {
            echo '<p>Value: '.$_GET['fcsfw_order_meta_field_nonce'].'</p>';
        }

        
        endif;
        echo '</div>';

        echo '
        <script type="text/javascript">
        jQuery(document).ready(function($) {
            $(".fcsfw_pickup_date").datepicker({
                dateFormat : "yy-mm-dd",
                minDate: 0  // Disallow past dates
            });
        });
        </script>';    
            echo '<style>
                #fcsfw_meta {
                 min-height: 150px;
                }
                @-webkit-keyframes dyinglight {
                    15% {
                      transform: scale(1.6);
                    }
                    50% {
                      transform: rotate(-89deg);
                    }
                    100% {
                      transform: rotate(-90deg);
                    }
                  }
                  
                  @keyframes dyinglight {
                    15% {
                      transform: scale(1.6);
                    }
                    50% {
                      transform: rotate(-89deg);
                    }
                    100% {
                      transform: rotate(-90deg);
                    }
                  }
                  .dl {
                    position: absolute;
                    left: 50%;
                    top: 50%;
                    transform: translate(-50%, -50%);
                    display: inline-block;
                  }
                  
                  .dl__square {
                    display: block;
                    width: 50px;
                    height: 50px;
                    background: #Ad8762;
                  }
                  
                  .dl__container {
                    position: absolute;
                    left: 0;
                    top: 0;
                    bottom: 0;
                    right: 0;
                    transform-origin: 50% 50% 0;
                    -webkit-animation: dyinglight 1s ease infinite;
                            animation: dyinglight 1s ease infinite;
                    -webkit-backface-visibility: hidden;
                            backface-visibility: hidden;
                  }
                  
                  .dl__corner--top:before,
                  .dl__corner--top:after,
                  .dl__corner--bottom:before,
                  .dl__corner--bottom:after {
                    position: absolute;
                    width: 17px;
                    height: 17px;
                    color: #2271b1;
                    content: "";
                  }
                  
                  .dl__corner--top:before {
                    border-left: 1px solid;
                    border-top: 1px solid;
                    top: -6px;
                    left: -6px;
                  }
                  .dl__corner--top:after {
                    border-right: 1px solid;
                    border-top: 1px solid;
                    top: -6px;
                    right: -6px;
                  }
                  
                  .dl__corner--bottom:before {
                    border-left: 1px solid;
                    border-bottom: 1px solid;
                    bottom: -6px;
                    left: -6px;
                  }
                  .dl__corner--bottom:after {
                    border-right: 1px solid;
                    border-bottom: 1px solid;
                    bottom: -6px;
                    right: -6px;
                  }
                </style>';
        //fcsfw_print_array($ShipmentRequest_Response);


        //fcsfw_print_array($flavorcloud_shipping_rate_id);


    }

//    
// Save the data of the Meta field ON POST SAVE
//
add_action( 'save_post', 'fcsfw_save_wc_order_fields', 10, 1 );
    function fcsfw_save_wc_order_fields( $post_id ) {

        // We need to verify this with the proper authorization (security stuff).

        // Check if our nonce is set.
        if ( ! isset( $_POST[ 'fcsfw_order_meta_field_nonce' ] ) ) {
            return $post_id;
        }
        $nonce = $_REQUEST[ 'fcsfw_order_meta_field_nonce' ];

        //Verify that the nonce is valid.
        if ( ! wp_verify_nonce( $nonce ) ) {
            return $post_id;
        }

        // If this is an autosave, our form has not been submitted, so we don't want to do anything.
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return $post_id;
        }

        // Check the user's permissions.
        if ( 'page' == $_POST[ 'post_type' ] ) {

            if ( ! current_user_can( 'edit_page', $post_id ) ) {
                return $post_id;
            }
        } else {

            if ( ! current_user_can( 'edit_post', $post_id ) ) {
                return $post_id;
            }
        }
        // --- Its safe for us to save the data ! --- //

        // Sanitize user input  and update the meta field in the database.
        update_post_meta( $post_id, '_fcsfw_pickup_date', $_POST[ 'fcsfw_pickup_date' ] );
    }


/**
 * Add General Settings tab under Woocommerce -> Shipping 
 */
add_filter( 'woocommerce_get_sections_shipping' , 'fcsfw_add_settings_tab' );
function fcsfw_add_settings_tab( $settings_tab ){
     $settings_tab['flavor_cloud_settings'] = __( 'Flavor Cloud Settings' );
     return $settings_tab;
}

/**
 * Add General Settings tab fields
 */
add_filter( 'woocommerce_get_settings_shipping' , 'fcsfw_get_settings' , 10, 2 );
function fcsfw_get_settings( $settings, $current_section ) {
    $custom_settings = array();

    if( 'flavor_cloud_settings' == $current_section ) {

        $custom_settings =  array(
            array(
                'name' => __( 'General Settings' ),
                'type' => 'title',
                'desc' => __( 'appID and restAPIkey Can be found in your Flavor Cloud account, Please click this link <a target="_blank" href="https://app.flavorcloud.com/fc/settings/api">https://app.flavorcloud.com/fc/settings/api</a> and go to API KEYS & API and create a new application.' ),
                'id'   => 'fcsfw_general_settings' 
            ),
            array(
                'name' => __( 'appID' ),
                'type' => 'text',
                'desc' => __( 'Flavor Cloud appID'),
                'desc_tip' => true,
                'id'	=> 'fcsfw_appID'
            ),
            array(
                'name' => __( 'restAPIKey' ),
                'type' => 'text',
                'desc' => __( 'Flavor Cloud restAPIKey'),
                'desc_tip' => true,
                'id'	=> 'fcsfw_restAPIKey'
            ),
            array(
                'name' => __( 'TermsOfTrade' ),
                'type' => 'select',
                'desc' => __( 'Terms of trade can be Delivery Duty Paid (DDP) or Delivery Duty Unpaid (DDU)'),
                'desc_tip' => true,
                'id'	=> 'fcsfw_TermsOfTrade',
                'options' => array(
                            'DDP' => __( 'DDP' ),
                            'DDU' => __('DDU')
                )
            ),
            array(
                'name' => __( 'Flavor Cloud Settingsh' ),
                'type' => 'fcsfw_termsoftrade',
                'id'   => 'free_shippingk' 
            ),
            array( 
                'type' => 'sectionend', 'id' => 'fcsfw_general_settings' 
            ),
            array(
                'name' => __( 'Ship From location' ),
                'type' => 'title',
                'desc' => __( 'Store address details. To edit the address, go to <a href="'.get_site_url().'/wp-admin/admin.php?page=wc-settings&tab=general">WooCommerce -> Settings -> General</a>.'),
                'id'   => 'fcsfw_shipping_location'
            ),
            array(
                'name' => __( 'Ship From Phone Number' ),
                'type' => 'text',
                'desc' => __( 'The Phone number that will be shown on the shipping label'),
                'desc_tip' => true,
                'default' => '18000000000', // Default value added here
                'id' => 'fcsfw_shipping_phone'
            ),
            array(
                'title' => __( 'Store Address' ),
                'desc' => '',
                'type' => 'fcsfw_display_address',
                'id'   => 'fcsfw_store_address'
            ),
            array( 
                'type' => 'sectionend', 'id' => 'fcsfw_shipping_location'
            ),
            array(
                'name' => __( 'Shipping Method Settings' ),
                'type' => 'title',
                'desc' => __( 'This Label will be displayed on the checkout page.' ),
                'id'   => 'fcsfw_shipping_methods' 
            ),
            array(
                'name' => __( 'Standard Label' ),
                'type' => 'text',
                'desc' => __( 'Change the default shipping label for Standard Shipping Method'),
                'desc_tip' => true,
                'id'	=> 'fcsfw_standard_label'
            ),
            array(
                'name' => __( 'Disable Standard' ),
                'type' => 'checkbox',
                'desc' => __( 'Check to disable the standard shipping method.'),
                'id'	=> 'fcsfw_standard_disable'
            ),
            array(
                'name' => __( 'Express Label' ),
                'type' => 'text',
                'desc' => __( 'Change the default shipping label for Express Shipping Method'),
                'desc_tip' => true,
                'id'	=> 'fcsfw_express_label'
            ),
            array(
                'name' => __( 'Disable Express' ),
                'type' => 'checkbox',
                'desc' => __( 'Check to disable the express shipping method.'),
                'id'	=> 'fcsfw_express_disable'
            ),
            array(
                'name' => __( 'Reverse Shipping Method Order' ),
                'type' => 'checkbox',
                'desc' => __( 'Check to reverse the order of shipping methods shown.' ),
                'id'	=> 'fcsfw_reverse_order'
            ),
            array( 
                'type' => 'sectionend', 'id' => 'fcsfw_shipping_methods' 
            ),
            array(
                'name' => __('Shipping Boxes'),
                'type' => 'title',
                'id'   => 'fcsfw_shipping_boxes',
            ),
            array(
                'name' => __('Boxes'),
                'type' => 'fcsfw_shipping_boxes',
                'id'   => 'fcsfw_shipping_boxes',
            ),
            array( 
                'type' => 'sectionend', 'id' => 'fcsfw_shipping_boxes'
            ),
            array( 
                'type' => 'sectionend', 
                'id' => 'fcsfw_shipping_boxes' 
            ),
            array(
                'name' => __( 'Debug' ),
                'type' => 'title',
                'desc' => __( 'Enable or disable the debug logs.' ),
                'id'   => 'fcsfw_debug' 
            ),
            array(
                'name' => __( 'Enable Debug Log' ),
                'type' => 'checkbox',
                'desc' => __( 'Check to enable the debug log. You can view the logs by navigating to <a href="'.get_site_url().'/wp-admin/admin.php?page=wc-status&tab=logs&log_file=woocommerce-flavorcloud-shipping-'.date("Y-m-d").'-log">WooCommerce -> Status -> Logs</a> and selecting the "woocommerce-flavorcloud-shipping" log.' ),
                'id' => 'fcsfw_enable_debug_log'
            ),
            array( 
                'type' => 'sectionend', 
                'id' => 'fcsfw_debug' 
            )
        );

        return $custom_settings;
    } else {
        return $settings;
    }
}


function fcsfw_shipping_boxes_setting() {
    // Get the boxes from the database
    //delete_option('fcsfw_shipping_boxes');

    ?>
    <style>
     .fcsfw_box, .fcsfw_setting_box {
         margin: 1em 0;
         padding: 1em;
         border: 1px solid #eee;
         border-radius: 5px;
         box-shadow: 0 2px 8px rgba(0,0,0,0.1);
         max-width: 800px;
         display: flex;
         flex-wrap: wrap;
         align-items: center;
         justify-content: space-between;
         background-color: #fff;
     }
     .fcsfw_setting_box {
        display: block;
        max-width: 600px;
     }
     .fcsfw_box label {
         width: calc(33% - 1em);
         margin-bottom: 1em;
         display: flex;
         flex-direction: column;
         font-size: 0.9em;
         color: #333;
     }
     .fcsfw_setting_box label {
         width: 100%;
     }
     .fcsfw_box label input {
         width: 100%;
         padding: 0.5em;
         border: 1px solid #ccc;
         border-radius: 3px;
         font-size: 0.9em;
     }
     .fcsfw_box input {
        width: auto !important;
     }
     .fcsfw_box button {
         width: 100%;
         margin-top: 1em;
         color: #fff;
         background-color: #333;
         border: none;
         border-radius: 3px;
         padding: 0.7em 0;
         font-size: 0.9em;
         cursor: pointer;
         transition: background-color 0.2s ease;
     }
     .fcsfw_box button:hover {
         background-color: #444;
     }
     .fcsfw_add {
         display: inline-block;
         margin-top: 1em;
         color: #fff;
         background-color: #4CAF50;
         border: none;
         border-radius: 3px;
         padding: 0.7em 1em;
         font-size: 0.9em;
         cursor: pointer;
         transition: background-color 0.2s ease;
     }
     .fcsfw_add:hover {
         background-color: #5ad365;
     }
     #fcsfw_fcsfw_shipping_boxes {
        background: #fff;
        padding: 40px;
        width: auto;
        display: inline-block;
        border-radius: 5px;
        box-shadow: 0 2px 8px rgba(0,0,0,0.1);
     }
 </style>
     <?php

        // Get the boxes from the database
        $boxes = get_option('fcsfw_shipping_boxes', array());
        // Check if there are no boxes and if so, use the default boxes
        if (empty($boxes)) {
            $weight_unit = get_option( 'woocommerce_weight_unit');
            if ($weight_unit == 'lbs') {
                $weight_unit = 'lb';
            }
            $dimension_unit = get_option( 'woocommerce_dimension_unit');

            

            // Default boxes (Do Not Edit)
            $default_boxes = array(
                0 => array(
                    'Name' => "S mailing carton",
                    'Length' => fcsfw_convertUnits(12,'in',$dimension_unit),
                    'Width' => fcsfw_convertUnits(10,'in',$dimension_unit),
                    'Height' => fcsfw_convertUnits(8,'in',$dimension_unit),
                    'Weight' => fcsfw_convertUnits(0.3,'oz',$weight_unit),
                    'MaxWeight' => fcsfw_convertUnits(20,'oz',$weight_unit)
                ),
                1 => array(
                    'Name' => "Custom Box",
                    'Length' => fcsfw_convertUnits(8,'in',$dimension_unit),
                    'Width' => fcsfw_convertUnits(6,'in',$dimension_unit),
                    'Height' => fcsfw_convertUnits(4,'in',$dimension_unit),
                    'Weight' => fcsfw_convertUnits(0.3,'oz',$weight_unit),
                    'MaxWeight' => fcsfw_convertUnits(20,'oz',$weight_unit)
                ),
                3 => array(
                    'Name' => "Large Box",
                    'Length' => fcsfw_convertUnits(12,'in',$dimension_unit),
                    'Width' => fcsfw_convertUnits(12,'in',$dimension_unit),
                    'Height' => fcsfw_convertUnits(5.5,'in',$dimension_unit),
                    'Weight' => fcsfw_convertUnits(0.3,'oz',$weight_unit),
                    'MaxWeight' => fcsfw_convertUnits(20,'oz',$weight_unit)
                ),
                4 => array(
                    'Name' => "Medium Box 1, Top Loading",
                    'Length' => fcsfw_convertUnits(11,'in',$dimension_unit),
                    'Width' => fcsfw_convertUnits(8.5,'in',$dimension_unit),
                    'Height' => fcsfw_convertUnits(5.5,'in',$dimension_unit),
                    'Weight' => fcsfw_convertUnits(0.3,'oz',$weight_unit),
                    'MaxWeight' => fcsfw_convertUnits(20,'oz',$weight_unit)
                ),
                8 => array(
                    'Name' => "Medium Box 2",
                    'Length' => fcsfw_convertUnits(16,'in',$dimension_unit),
                    'Width' => fcsfw_convertUnits(12,'in',$dimension_unit),
                    'Height' => fcsfw_convertUnits(7,'in',$dimension_unit),
                    'Weight' => fcsfw_convertUnits(0.3,'oz',$weight_unit),
                    'MaxWeight' => fcsfw_convertUnits(20,'oz',$weight_unit)
                ),
                7 => array(
                    'Name' => "Small Box",
                    'Length' => fcsfw_convertUnits(8.63,'in',$dimension_unit),
                    'Width' => fcsfw_convertUnits(5.38,'in',$dimension_unit),
                    'Height' => fcsfw_convertUnits(1.63,'in',$dimension_unit),
                    'Weight' => fcsfw_convertUnits(0.3,'oz',$weight_unit),
                    'MaxWeight' => fcsfw_convertUnits(20,'oz',$weight_unit)
                ),
            );
            
            
            

        $boxes = $default_boxes;
        // Save the default boxes to the database
        update_option('fcsfw_shipping_boxes', $default_boxes);
        }

     //echo print_r($boxes,true);
     $weight_unit = get_option( 'woocommerce_weight_unit' );
     $dimension_unit = get_option( 'woocommerce_dimension_unit' );

    echo '<tr valign="top">';
    echo '<td class="forminp" id="fcsfw_fcsfw_shipping_boxes">';
    echo '<p>Weight Unit: <strong>'.$weight_unit.'</strong></p>';
    echo '<p>Dimension Unit: <strong>'.$dimension_unit.'</strong></p>';
    echo '<p>To adjust units, navigate to <a href="'.get_site_url().'/wp-admin/admin.php?page=wc-settings&tab=products">WooCommerce -> Settings -> Products</a></p><br>';
    echo '<p>Input dimensions for all your shipping boxes using the above units.</p>';
    echo '<p>This crucial step allows the plugin to calculate the most economical shipping rate for your customers by simulating the packing of the items in their shopping cart. This optimizes costs and delivers the best shipping options.</p>';
    echo '<button type="button" class="fcsfw_add">Add Shipping Box</button><br>';
    echo '<br><small>NOTES:<br> The order of the boxes does not matter. The plugin will select the smallest box that can accommodate all items in the cart once a shipping date is set.</small><br>';
    echo '<small>The Weight field represents the weight of the empty box, while MaxWeight is the maximum weight that the box can accommodate (set a higher weight if uncertain).</small><br>';
    echo '<div class="fcsfw_shipping_boxes">';


     // Box index
     $i = 0;
     
     // Check if there are boxes
     if (!empty($boxes)) {
         foreach ($boxes as $box) {
             $name = isset($box['Name']) ? esc_attr($box['Name']) : '';
             $length = isset($box['Length']) ? esc_attr($box['Length']) : '';
             $width = isset($box['Width']) ? esc_attr($box['Width']) : '';
             $height = isset($box['Height']) ? esc_attr($box['Height']) : '';
             $weight = isset($box['Weight']) ? esc_attr($box['Weight']) : '';
             $maxWeight = isset($box['MaxWeight']) ? esc_attr($box['MaxWeight']) : '';
 
             echo '<div class="fcsfw_box">';
             echo '<label>Name: <input type="text" name="fcsfw_shipping_boxes['.$i.'][Name]" value="' . $name . '" ></label>';
             echo '<label>Length: <input type="text" name="fcsfw_shipping_boxes['.$i.'][Length]" value="' . $length . '" class="fcsfw_numeric-input"></label>';
             echo '<label>Width: <input type="text" name="fcsfw_shipping_boxes['.$i.'][Width]" value="' . $width . '"class="fcsfw_numeric-input" ></label>';
             echo '<label>Height: <input type="text" name="fcsfw_shipping_boxes['.$i.'][Height]" value="' . $height . '"class="fcsfw_numeric-input" ></label>';
             echo '<label>Weight: <input type="text" name="fcsfw_shipping_boxes['.$i.'][Weight]" value="' . $weight . '"class="fcsfw_numeric-input" ></label>';
             echo '<label>MaxWeight: <input type="text" name="fcsfw_shipping_boxes['.$i.'][MaxWeight]" value="' . $maxWeight . '"></label>';
             echo '<button class="fcsfw_remove">Remove</button>';
             echo '</div>';
             $i++;
         }
     }
     
     echo '</div>';

     echo '</td>';
     echo '</tr>';
 }
add_action('woocommerce_admin_field_fcsfw_shipping_boxes', 'fcsfw_shipping_boxes_setting');




add_action( 'woocommerce_admin_field_fcsfw_display_address', 'output_fcsfw_display_address' );
function output_fcsfw_display_address() {
    $store_address = get_option( 'woocommerce_store_address' );
    $store_address_2 = get_option( 'woocommerce_store_address_2' );
    $store_city = get_option( 'woocommerce_store_city' );
    $store_country_state = get_option( 'woocommerce_default_country' );
    $store_postcode = get_option( 'woocommerce_store_postcode' );

    // Assigning class "add" to the message
    $msg = '<span class="fcsfw_setting_error">Required, Please add it to the Woocommerce General Settings.</span>';

    // Wrapping the content within a div with class "box"
    echo '<div class="fcsfw_setting_box">';
    // Applying class "label" to each paragraph
    echo '<p class="label"><strong>Address 1:</strong> ' . (!empty($store_address) ? $store_address : $msg) . '</p>';
    echo '<p class="label"><strong>Address 2:</strong> ' . (!empty($store_address_2) ? $store_address_2 : 'N/A') . '</p>';
    echo '<p class="label"><strong>City:</strong> ' . (!empty($store_city) ? $store_city : $msg) . '</p>';
    echo '<p class="label"><strong>Country / State:</strong> ' . (!empty($store_country_state) ? $store_country_state : $msg) . '</p>';
    echo '<p class="label"><strong>Postcode / ZIP:</strong> ' . (!empty($store_postcode) ? $store_postcode : $msg) . '</p>';
    echo '</div>';  // Closing the div with class "box"
}





/**
 * Custom Fields 
 */
add_action( 'woocommerce_admin_field_fcsfw_termsoftrade', 'output_fcsfw_termsoftrade' );
function output_fcsfw_termsoftrade( ) {
    echo '<p>When TermsOfTrade DDP is selected, international customers will get charged duty and tax fees on checkout.</p>';
}

/**
 * Load Admin JS
 */
function fcsfw_load_custom_wp_admin_style($hook) {
    // Check if we're in the right page
    if ('woocommerce_page_wc-settings' !== $hook) {
        return;
    }
    // Check the correct parameters in the URL
    if (isset($_GET['section']) && 'flavor_cloud_settings' == $_GET['section'] ) {
        wp_register_script('fcsfw_wp_admin_js', plugin_dir_url(__FILE__) . '/js/fcsfw-admin.js', array('jquery'), '1.1.8', true);
        wp_enqueue_script('fcsfw_wp_admin_js');
    }
}
add_action('admin_enqueue_scripts', 'fcsfw_load_custom_wp_admin_style');
