<?php
/**
 * FlavorCloud Shipping For Woocommerce API Main
 */

/**
 * Increase http request timeout from 5 to 15 sec, The flavor Cloud Api sometimes takes more than 5 sec.
 */
function fcsfw_http_request_timeout( ) {
    return 25;
}
add_filter( 'http_request_timeout', 'fcsfw_http_request_timeout' );


/**
 * returns true if $needle is a substring of $haystack
 */
function fcsfw_contains($needle, $haystack)
{
    return strpos($haystack, $needle) !== false;
}


/**
 * Function to get settings before posting to the Flavor Cloud API
 */
//'TermsOfTrade' => 'DDP' == Duties Paid
//'TermsOfTrade' => 'DDU' == Duties Un Paid
function FlavorCloud_API_Get_Settings($data) {

    if ($data == 'API') {
        $units = fcsfw_get_woocommerce_weight_and_dimension_units();
        $api = array(
            'appID' => get_option( 'fcsfw_appID' ),
            'restAPIKey' => get_option( 'fcsfw_restAPIKey' ),
            'weight_unit' => $units['weight_unit'],  // Unit for weight of the shipment. Possible values are - KG|LB
            'dimensionunit' => $units['dimension_unit'], // Unit for dimensions of the shipment. Possible values are - IN|CM
            'TermsOfTrade' => get_option( 'fcsfw_TermsOfTrade' ),
            'fcsfw_standard_label' => get_option( 'fcsfw_standard_label' ),
            'fcsfw_standard_disable' => get_option( 'fcsfw_standard_disable' ),
            'fcsfw_express_label' => get_option( 'fcsfw_express_label' ),
            'fcsfw_express_disable' => get_option( 'fcsfw_express_disable' ),
            'fcsfw_reverse_order' => get_option( 'fcsfw_reverse_order' ),
            'Currency' => 'USD',
            'endpoints' => array (
                'Rates' => 'https://partnerapi.flavorcloud.com/Rates',
                'ShipmentRequest' => 'https://partnerapi.flavorcloud.com/Shipments',
            )
        );
        return $api;
    }

    if ($data == 'ShipFromAddress') {

        $WC_Countries = new WC_Countries();
        //fcsfw_print_array($WC_Countries->get_base_address());
        //die();

        $ShipFromAddress = array(
            'Name' => get_bloginfo('site_title'),
            'AttentionName' => '',
            'AddressLine1' => $WC_Countries->get_base_address(),
            'AddressLine2' => $WC_Countries->get_base_address_2(),
            'AddressLine3' => '',
            'City' => $WC_Countries->get_base_city(),
            'State' => $WC_Countries->get_base_state(),
            'Country' => $WC_Countries->get_base_country(),
            'Zip' => $WC_Countries->get_base_postcode(),
            'Phone' => get_option( 'fcsfw_shipping_phone' ),  // fcsfw_shipping_phone
            'Email' => get_bloginfo('admin_email')
        );
        return $ShipFromAddress;
    }
}


/**
 * Function to build pieces Array, or return items for BoxPacking
 */
function FlavorCloud_API_Pieces($order, $type = 'Shipments') {    
    $pieces = array();
    $items = array();

    // Get the current WooCommerce settings
    $units = fcsfw_get_woocommerce_weight_and_dimension_units();
    $weight_unit_api = $units['weight_unit'];
    $dimension_unit_api = $units['dimension_unit'];

    $weight_unit = get_option( 'woocommerce_weight_unit');
    if ($weight_unit == 'lbs') {
        $weight_unit = 'lb';
    }
    $dimension_unit = get_option( 'woocommerce_dimension_unit');


    // Get and Loop Over Order Items
    foreach ( $order->get_items() as $item_id => $item ) {

        $item_discounted_price = $item->get_total() / $item->get_quantity();
        $product = $item->get_product();
        if( $product->is_type('variation') ){ 
            $product_id = $item->get_variation_id();
        } else {
            $product_id = $item->get_product_id();
        }


        // Convert the product dimensions and weight to the correct units 0.01
        $converted_weight = fcsfw_convertUnits($product->get_weight(),$weight_unit,strtolower($weight_unit_api));
        // If our product weight is too low set it to 0.01. Ex: if product is 1grams then the above will return 0KG (10grams will return 0.01)
        if($converted_weight == 0) {
        $converted_weight = 0.01;
        }
        $converted_length = fcsfw_convertUnits($product->get_length(),$dimension_unit,strtolower($dimension_unit_api));
        $converted_width = fcsfw_convertUnits($product->get_width(),$dimension_unit,strtolower($dimension_unit_api));
        $converted_height = fcsfw_convertUnits($product->get_height(),$dimension_unit,strtolower($dimension_unit_api));

        $pieces[] = array(
            'Quantity' => (float)$item->get_quantity(),
            'Weight' => $converted_weight,
            'Length' => $converted_length,
            'Width' => $converted_width,
            'Height' => $converted_height,
            'SalePrice' => (float)$item_discounted_price,
            'HSCode' => $product->get_meta('_hscode'),
            'OriginCountryCode' => $product->get_meta('_origincountrycode'),
            'Material' => $product->get_meta('_material'),
            'Description' => $item->get_name(),
            'SKU' => $product->get_SKU(),
            'Category' =>  array()
        );
        $items[] = array(
            'Name' => $item->get_name(),
            'Width' => $converted_width,
            'Length' => $converted_length,
            'Height' => $converted_height,
            'Weight' => $converted_weight,
            'Quantity' => (float)$item->get_quantity(),
            'Price' => (float)$item_discounted_price,
            'SKU' => $product->get_SKU(),
        );
    }

    //update_post_meta( $order->get_id(), '_testing_fc_pieces', $pieces );
    //update_post_meta( $order->get_id(), '_testing_fc_items', $items);
    
    if ($type == 'Shipments') {
        $box = FlavorCloud_API_BinPack( FlavorCloud_API_Pieces($order, $type = 'items') );

        $converted_weight_box = fcsfw_convertUnits($box['Weight'],$weight_unit,strtolower($weight_unit_api));
        if($converted_weight_box == 0) {
            $converted_weight_box = 0.01;
        }
        $converted_length_box = fcsfw_convertUnits($box['Length'],$dimension_unit,strtolower($dimension_unit_api));
        $converted_width_box = fcsfw_convertUnits($box['Width'],$dimension_unit,strtolower($dimension_unit_api));
        $converted_height_box = fcsfw_convertUnits($box['Height'],$dimension_unit,strtolower($dimension_unit_api));

        $Shipments = array(
            array(
                "Piece" => $pieces,
                "Package" => array(
                    "Weight" => $converted_weight_box,
                    "Length" => $converted_length_box,
                    "Width" => $converted_width_box,
                    "Height" => $converted_height_box,
                    "Reference" => $box['Name']
                )
            )
        );
        

        update_post_meta( $order->get_id(), '_testing_fc_box', $box);
        //die();
        return $Shipments;
    } else {
        //update_post_meta( $order->get_id(), '_testing_fc_box', 'no');
        //die();
        return $items;
    }

    //die();
}

/*
function FlavorCloud_API_Pieces($order, $type = 'Shipments') {    
    $pieces = array();
    $items = array();
     // Get and Loop Over Order Items
    foreach ( $order->get_items() as $item_id => $item ) {
        //var_dump($item->get_total());
        //var_dump($item->get_quantity());
        $item_discounted_price = $item->get_total() / $item->get_quantity();
        //var_dump($item_discounted_price);
        // Get an instance of the WC_Product object (can be a product variation  too)
        $product = $item->get_product();
        if( $product->is_type('variation') ){ 
            $product_id = $item->get_variation_id();
            //fcsfw_print_array($item->get_variation_id());
        } else {
            $product_id = $item->get_product_id();
            //fcsfw_print_array($item->get_product_id());
        }
        //fcsfw_print_array($product->get_weight());

        // Now we need convert the currentley selected weight under woocommerce products to KG then add it to the quote. 

        $pieces[] = array(
            'Quantity' => (float)$item->get_quantity(),
            'Weight' => (float)$product->get_weight(),
            'Length' => (float)$product->get_length(),
            'Width' => (float)$product->get_width(),
            'Height' => (float)$product->get_height(),
            'SalePrice' => (float)$item_discounted_price,
            'HSCode' => $product->get_meta('_hscode'),
            'OriginCountryCode' => $product->get_meta('_origincountrycode'),
            'Material' => $product->get_meta('_material'),
            'Description' => $item->get_name(),
            'SKU' => $product->get_SKU(),
            'Category' =>  array()
        );
        // For Box Packer
        $items[] = array(
            'Name' => $item->get_name(),
            'Width' => (float)$product->get_width(),
            'Length' => (float)$product->get_length(),
            'Height' => (float)$product->get_height(),
            'Weight' => (float)$product->get_weight(),
            'Quantity' => (float)$item->get_quantity(),
            'Price' => (float)$item_discounted_price,
            'SKU' => $product->get_SKU(),
        );
    }
    
    //fcsfw_print_array( $pieces);

    if ($type == 'Shipments') {
        $box = FlavorCloud_API_BinPack( FlavorCloud_API_Pieces($order, $type = 'items') );
        //fcsfw_print_array($box);
        $Shipments = array(
            array(
                "Piece" => $pieces,
                "Package" => array(
                    "Weight" => (float)$box['Weight'],
                    "Length" => (float)$box['Length'],
                    "Width" => (float)$box['Width'],
                    "Height" => (float)$box['Height'],
                    "Reference" => $box['Name']
                )
            )
        );
        return $Shipments;
    } else {
        return $items;
    }
}
*/



    /**
 * Pack the cart items in a Box (Bin Packing)
 * Inches / lb 
 * @param array $items
 */
function FlavorCloud_API_BinPack( $items ='' ) {
    $boxes = get_option('fcsfw_shipping_boxes', array());
    //fcsfw_print_array($boxes);
    // Load Packing Scripts 
    require_once FCSFW_PLUGIN_DIR_PATH . 'packing/box.php';
    require_once FCSFW_PLUGIN_DIR_PATH . 'packing/item.php';
    require FCSFW_PLUGIN_DIR_PATH . 'packing/flavorcloud-packer.php';

    /*
    FlavorCloud_API_BinPack( FlavorCloud_API_Pieces($order, $type = 'items') );
    $Package = array(
        "Weight" => 0,
        "Length" => 0,
        "Width" => 0,
        "Height" => 0,
        "Reference" => "string"
    );
    return $quote_box;
    */
    return $quote_box;
}


/**
 * Function to call the Flavor Cloud API's ShipmentRequest method. 
 */
function FlavorCloud_API_ShipmentRequest($post_id, $pickup_date='') {

    $order = wc_get_order( $post_id );

    $main_data = FlavorCloud_API_Get_Settings('API');
    $endpoint = $main_data['endpoints']['ShipmentRequest'];
    $flavorcloud_shipping_rate_id = explode("_", get_post_meta( $post_id ,'flavorcloud_shipping_rate_id', true));
    $ServiceCode = strtoupper($flavorcloud_shipping_rate_id[0]);
    $HashKey = $flavorcloud_shipping_rate_id[1];
    //$DutyHashKey = $flavorcloud_shipping_rate_id[2];
    // Check if DutyHashKey exists, if not assign default value
    $DutyHashKey = isset($flavorcloud_shipping_rate_id[2]) ? $flavorcloud_shipping_rate_id[2] : ''; 
    // Build Pieces Array
    //FlavorCloud_API_Pieces($order);
    $quote_box = FlavorCloud_API_BinPack( FlavorCloud_API_Pieces($order, $type = 'items') );


    //fcsfw_print_array($order);
   // die();


    $create_label = array(
        "AppID"=> $main_data['appID'], // *
        "RestApiKey"=> $main_data['restAPIKey'], // *
        "CustomerKey"=> "", // *   // <-- customer_key Specify the customer key for whom the shipment is getting created. (Applicable only for the 3PL user)
        "HashKey"=> $HashKey,  // * <- Specify a hashkey returned from rate api
        "DutyHashKey"=> $DutyHashKey,
        "PickUpDate"=> $pickup_date,  // <-- Shipment PickUpDate (ISO Date) in text format YYYY-MM-DD; EX: 2021-04-06
        "ShipDateStamp"=> $pickup_date,
        "Async"=> false, // <- OPTIONAL
        "Reference"=> $post_id, // * <- This field is for your reference., A good use of it would be to provide your order number here.
        "LabelFormat"=> "PDF", // <-Accepted values are - PDF|ZPL. Default PDF
        "WeightUnit"=> $main_data['weight_unit'],  // <-- Unit for weight of the shipment. Possible values are - KG|LB
        "Currency"=> $main_data['Currency'],  // <-- currency for the rate
        "DimensionUnit"=> $main_data['dimensionunit'],  // <-- Unit for dimensions of the shipment. Possible values are - IN|CM
        "Insurance"=> "N",
        "ServiceCode"=> $ServiceCode, // * <-- !!!Need to corelate the shipping method selected with this!!!!   service level/name. Accepted values are - 'express' | 'standard' | 'economy'. And array of service levels also accepted like ['express', 'standard', 'economy']
        "TermsOfTrade"=> "DDP", // <-- Terms of trade can be Delivery Duty Paid (DDP) or Delivery Duty Unpaid (DDU)
        "IsReturn"=> "N",
        "ShipmentKey"=> $post_id,
        "ShipFromAddress"=> FlavorCloud_API_Get_Settings('ShipFromAddress'),
        "ShipToAddress"=> array(
          "Name"=> $order->get_shipping_first_name().' '.$order->get_shipping_last_name(),
          "AttentionName"=> "",
          "AddressLine1"=> $order->get_shipping_address_1(),
          "AddressLine2"=> $order->get_shipping_address_2(),
          "AddressLine3"=> "",
          "City"=> $order->get_shipping_city(),
          "State"=> $order->get_shipping_state(),
          "Country"=> $order->get_shipping_country(),
          "Zip"=> $order->get_shipping_postcode(),
          "Phone"=> $order->get_billing_phone(),
          "Email"=> $order->get_billing_email(),
          "FederalTaxId"=> "",
          "StateTaxId"=> ""
        ),
        "Shipments"=> FlavorCloud_API_Pieces($order, 'Shipments'),
        "ReasonForExport"=> "sale" 
      );

    $arg_header = array(
        'content-type' => 'application/json',
        'accept' => 'application/json',
    );


    //error_log( print_r( $quote_box, true ) );
    //error_log( print_r( json_encode($quote_box), true ) );
    //error_log( print_r( FlavorCloud_API_Pieces($order, $type = 'items'), true ) );
    //die();
    //update_post_meta( '7319', '_Cloud_Flare_ShipmentRequest_Box', '{"Name":"Priority Mail Box 4","Length":7.25,"Width":7.25,"Height":6.5,"Weight":0.3,"MaxWeight":20}' );
    //fcsfw_print_array($create_label);
    //die();
    
    //var_dump($create_label);
    //die();

    // Make The Api Call 
    $response = wp_remote_post($endpoint, array(
        'headers'   => $arg_header,
        'body'      => json_encode($create_label),
        'method'    => 'POST'
    ));

    //var_dump($response);
    //die();
    //SHIPDATESTAMP.FORMAT.INVALID
    //$this->debug( fcsfw_print_array(fcsfw_print_array($response)));
    //fcsfw_logger($response);
    //die();
    /*
    $response = array (
        'ShipmentID' => "knzfejxpfq",
        'Reference' => "7315",
        'TrackingNumber' => "1688900078953",
        'LabelUrl' => array (
          "https://cdn.flavorcloud.com/s-731524071688900078953-1688900078953-label.pdf"
        ),
        'Carrier' => "APC LAX",
        'SubmittedElectronically' => true,
        'CustomsInvoiceURL' => "https://cdn.flavorcloud.com/s-731524071688900078953-731524071688900078953-invoice.pdf",
        'RootRequestId' => "ffa8e8c3-6338-4a5c-891f-d83124651a91"
    );
*/
    if ( is_wp_error( $response ) ) {
        //$this->debug( __( 'Cannot Create FlavorCloud Shippment ', 'woocommerce-shipping-flavorcloud' ) . $response->get_error_message(), 'error' );
        
        /*
        $responceData = array (
            'ShipmentID' => "knzfejxpfq",
            'Reference' => "7315",
            'TrackingNumber' => "1688900078953",
            'LabelUrl' => array (
              "https://cdn.flavorcloud.com/s-731524071688900078953-1688900078953-label.pdf"
            ),
            'Carrier' => "APC LAX",
            'SubmittedElectronically' => true,
            'CustomsInvoiceURL' => "https://cdn.flavorcloud.com/s-731524071688900078953-731524071688900078953-invoice.pdf",
            'RootRequestId' => "ffa8e8c3-6338-4a5c-891f-d83124651a91"
        );
        $responceData = $response;
  */
        // Save responce as post meta 
        //update_post_meta( $post_id, '_Cloud_Flare_ShipmentRequest_Response', json_encode($responceData) );


    } else {
        // Response body.
        $body = wp_remote_retrieve_body( $response );
        // On success ($body is a valid JSON string), output the shipping methods Array 
        $responceData = ( ! is_wp_error( $response ) ) ? json_decode( $body, true ) : null;
       /* $responceData = array (
            'ShipmentID' => "knzfejxpfq",
            'Reference' => "7315",
            'TrackingNumber' => "1688900078953",
            'LabelUrl' => array (
              "https://cdn.flavorcloud.com/s-731524071688900078953-1688900078953-label.pdf"
            ),
            'Carrier' => "APC LAX",
            'SubmittedElectronically' => true,
            'CustomsInvoiceURL' => "https://cdn.flavorcloud.com/s-731524071688900078953-731524071688900078953-invoice.pdf",
            'RootRequestId' => "ffa8e8c3-6338-4a5c-891f-d83124651a91"
        );*/
        if (is_array($responceData['error'])):
            update_post_meta( $post_id, '_Cloud_Flare_ShipmentRequest_Error', json_encode($responceData) );
            return json_encode($responceData['error']);
        else :
            // Add the response
            update_post_meta( $post_id, '_Cloud_Flare_ShipmentRequest_Request', json_encode($create_label) );
            update_post_meta( $post_id, '_Cloud_Flare_ShipmentRequest_Response', json_encode($responceData) );
            // Add the Shipping Box Dimentions
            update_post_meta( $post_id, '_Cloud_Flare_ShipmentRequest_Box', json_encode($quote_box) );
            return json_encode($responceData);
        endif;
    }
// wp_http_validate_url($ShipmentRequest_Response->CustomsInvoiceURL)
    //return json_encode($create_label);
    return json_encode($responceData);




      //return fcsfw_print_array($create_label);
      //return $create_label;
}
