<?php
/**
 * WC_Shipping_FlavorCloud class.
 *
 * @class 		WC_Shipping_FlavorCloud
 * @version		1.1.0
 * @package		FlavorCloud-for-WooCommerce/Classes
 * @category	Class
 * @author 		Qwebmaster
 */
class WC_Shipping_FlavorCloud extends WC_Shipping_Method {
    /**
     * FlavorCloud endpoint and Credentials.
     */
    private $endpoint = "https://partnerapi.flavorcloud.com/Rates";
    //public $appID = get_option( 'fcsfw_appID' );
    //private $restAPIKey = get_option( 'fcsfw_restAPIKey' );
    // Unit for weight of the shipment. Possible values are - KG|LB
    private $debug = "yes";
    // $this->restAPIKey
    
    /**
     * Constructor. The instance ID is passed to this.
     */

     
    public function __construct($instance_id = 0) {
        $this->id = "flavor-cloud";
        $this->instance_id = absint($instance_id);
        $this->method_title = __("FlavorCloud Shipping Method");
        $this->method_description = __("FlavorCloud Shipping method, Live API Shipping Rates.");
        $this->supports = ["shipping-zones", "instance-settings", "instance-settings-modal", ];
        //$this->instance_form_fields = ["enabled" => ["title" => __("Enable/Disable"), "type" => "checkbox", "label" => __("Enable this shipping method"), "default" => "yes", ], "title" => ["title" => __("Method Title"), "type" => "text", "description" => __("This controls the title prefix which the user sees during checkout."), "default" => __("Flavor Cloud"), "desc_tip" => true, ], ];
        $this->instance_form_fields = [
            "enabled" => [
                "title" => __("Enable/Disable"),
                "type" => "checkbox",
                "label" => __("Enable this shipping method"),
                "default" => "yes",
            ],
            "title" => [
                "title" => __("Method Title"),
                "type" => "text",
                "description" => __("This controls the title prefix which the user sees during checkout."),
                "default" => __("Flavor Cloud"),
                "desc_tip" => true,
            ],
            'standard_shipping_rate' => [
                'title'       => __( 'Flat Standard Shipping Rate', 'woocommerce' ),
                'type'        => 'text',
                'description' => __( 'Enter the fixed rate for standard shipping. This will override the shipping estimate from FlavorCloud', 'woocommerce' ),
                'default'     => '',
            ],
            'express_shipping_rate' => [
                'title'       => __( 'Flat Express Shipping Rate', 'woocommerce' ),
                'type'        => 'text',
                'description' => __( 'Enter the fixed rate for express shipping. This will override the shipping estimate from FlavorCloud', 'woocommerce' ),
                'default'     => '',
            ],
        ];
        
        $this->enabled = $this->get_option("enabled");
        $this->title = $this->get_option("title");
        $this->standard_shipping_rate = $this->get_option("standard_shipping_rate");
        $this->express_shipping_rate = $this->get_option("express_shipping_rate");
        $this->init();
        $this->units = fcsfw_get_woocommerce_weight_and_dimension_units();
        $this->weight_unit_api = $this->units["weight_unit"];
        $this->dimension_unit_api = $this->units["dimension_unit"];
        // Get the current WooCommerce settings
        $this->weight_unit = get_option("woocommerce_weight_unit");
        if ($this->weight_unit == "lbs") {
            $this->weight_unit = "lb";
        }
        $this->dimension_unit = get_option("woocommerce_dimension_unit");
        //add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
        // Hook the function to the standard cart fees calculation
        add_filter("woocommerce_cart_calculate_fees", [$this, "flavorcloud_add_additional_fees"], 99, 1);
        // Hook the function to the subscriptions recurring totals calculation
        add_filter("woocommerce_subscriptions_calculate_recurring_totals", [$this, "flavorcloud_add_additional_fees"], 99, 1);
    }
    /**
     * Initialization.
     *
     * @return void
     */
    private function init() {
        // Load the settings.
        //$this->init_form_fields();
        //$this->set_settings();
        add_action("woocommerce_update_options_shipping_" . $this->id, [$this, "process_admin_options", ]);
        add_action("woocommerce_update_options_shipping_" . $this->id, [$this, "clear_transients", ]);
    }
    /**
     * Output a debug message.
     *
     * @param string $message Debug message.
     * @param string $type    Message type.
     */
    public function debug($message, $type = "notice") {
        //if ( $this->debug || ( current_user_can( 'manage_options' ) && 'error' == $type ) ) {
        if ($this->debug || current_user_can("manage_options")) {
            wc_add_notice($message, $type);
        }
    }
    /**
     * Clear FlavorCloud transients.
     */
    public function clear_transients() {
        global $wpdb;
        $wpdb->query("DELETE FROM `$wpdb->options` WHERE `option_name` LIKE ('_transient_flavorcloud_quote_%') OR `option_name` LIKE ('_transient_timeout_flavorcloud_quote_%')");
    }
    /**
     * Get FlavorCloud Shipping Rates via API function.
     */
    public function Flavor_Cloud_API_GetRates($pieces, $quote_box, $ShipToAddress, $ShipFromAddress = "") {
        //var_dump($quote_box);
        $main_data = FlavorCloud_API_Get_Settings("API");
        $FLCLShippingServiceURL = $main_data["endpoints"]["Rates"];
        $converted_weight = fcsfw_convertUnits($quote_box["Weight"], $this->weight_unit, strtolower($this->weight_unit_api));
        // If our product weight is too low set it to 0.01. Ex: if product is 1grams then the above will return 0KG (10grams will return 0.01)
        if ($converted_weight == 0) {
            $converted_weight = 0.01;
        }
        //var_dump($converted_weight);
        $converted_width = fcsfw_convertUnits($quote_box["Width"], $this->dimension_unit, strtolower($this->dimension_unit_api));
        $converted_length = fcsfw_convertUnits($quote_box["Length"], $this->dimension_unit, strtolower($this->dimension_unit_api));
        $converted_height = fcsfw_convertUnits($quote_box["Height"], $this->dimension_unit, strtolower($this->dimension_unit_api));
        //fcsfw_print_array($main_data['TermsOfTrade']);
        //die();
        $args = ["AppID" => $main_data["appID"], "RestApiKey" => $main_data["restAPIKey"], "CustomerKey" => "", "Reference" => "1234567", "LabelFormat" => "PDF", "WeightUnit" => $this->weight_unit_api, "Currency" => "USD", "DimensionUnit" => $this->dimension_unit_api, "Insurance" => "N", "ServiceCode" => ["STANDARD", "EXPRESS", "STANDARD-ECONOMY"], "TermsOfTrade" => [$main_data["TermsOfTrade"]], "IsReturn" => "N", "IncludeLandedCost" => true, "ShipFromAddress" => FlavorCloud_API_Get_Settings("ShipFromAddress"), "ShipToAddress" => $ShipToAddress, "Pieces" => $pieces, "Package" => ["Length" => $converted_length, "Width" => $converted_width, "Height" => $converted_height, "Weight" => $converted_weight, ], "ReasonForExport" => "Sale", ];
        $arg_header = ["content-type" => "application/json", "accept" => "application/json", ];
        //fcsfw_print_array($args);
        //die();
        // Check Cache Before doing API Request
        // Build Transient, to be retreved later when the order is made.
        // The info will be used to create a shipping label
        $transient = "flavorcloud_quote_" . md5(json_encode($args));
        $cached_response = get_transient($transient);
        // ENABLE TESTING 2023
        //fcsfw_print_array($args);
        //die();
        // AIT = Advance Income Tax
        if (false === $cached_response) {
            $response = wp_remote_post($FLCLShippingServiceURL, ["headers" => $arg_header, "body" => json_encode($args), "method" => "POST", ]);
            //$this->debug( fcsfw_print_array(fcsfw_print_array($response)));
            //die();
            if (is_wp_error($response)) {
                //fcsfw_print_array( $response);
                //$this->debug( __( 'Cannot retrieve FlavorCloud rate: ', 'woocommerce-shipping-flavorcloud' ) . $response->get_error_message(), 'error' );
                //fcsfw_logger('Cannot retrieve FlavorCloud rate:'.$response->get_error_message());
            } else {
                //fcsfw_print_array( $response);
                // Response body.
                $body = wp_remote_retrieve_body($response);
                // On success ($body is a valid JSON string), output the shipping methods Array
                $responceData = !is_wp_error($response) ? json_decode($body, true) : null;
                set_transient($transient, $responceData, 12 * HOUR_IN_SECONDS);
                //]fcsfw_logger( var_dump($responceData));
                
            }
        } else {
            $responceData = $cached_response;
            //$this->debug( __( 'Flavor Cloud: Using cached response.', 'woocommerce-shipping-flavorcloud' ) );
            
        }
        return $responceData;
    }
    public function flavorcloud_add_additional_fees($cart) {
        global $woocommerce;
        // Access the $additional_fees variable outside of the function's scope
        $additional_fees = $GLOBALS["flavorcloud_additional_fees"];
        $chosen_shipping_methods = WC()->session->get("chosen_shipping_methods");
        if (fcsfw_contains("Standard", $chosen_shipping_methods[0])) {
            $cart->add_fee("Duty:", $additional_fees["Standard"]["Duty"], false, "standard");
            $cart->add_fee("Sales Tax:", $additional_fees["Standard"]["SalesTax"], false, "standard");
        } elseif (fcsfw_contains("Express", $chosen_shipping_methods[0])) {
            $cart->add_fee("Duty:", $additional_fees["Express"]["Duty"], false, "standard");
            $cart->add_fee("Sales Tax:", $additional_fees["Express"]["SalesTax"], false, "standard");
        } elseif (fcsfw_contains("Economy", $chosen_shipping_methods[0])) {
            $cart->add_fee("Duty:", $additional_fees["Economy"]["Duty"], false, "standard");
            $cart->add_fee("Sales Tax:", $additional_fees["Economy"]["SalesTax"], false, "standard");
        }
    }
    /**
     * Shipping method is available only when destination county,zipcity,street are loaded.
     *
     * @param array $package Package to ship.
     *
     * @return bool True if shipping method is available.
     */
    public function is_available($package) {
        if (empty($package["destination"]["country"]) || empty($package["destination"]["postcode"]) || empty($package["destination"]["city"]) || empty($package["destination"]["address"])) {
            return false;
        }
        return apply_filters("woocommerce_shipping_" . $this->id . "_is_available", true, $package);
    }
    /**
     * calculate_shipping function.
     * @param array $package (default: array())
     */
    public function calculate_shipping($package = []) {
        // Only return rates if the package has a destination including country.
        if ("" === $package["destination"]["country"]) {
            $this->debug(__("FlavorCloud: Country not supplied. Rates not requested.", "woocommerce-shipping-flavorcloud"));
            return;
        }
        //$this->debug( fcsfw_print_array($package['contents']));
        // Loop through shipping packages
        $ctr = 0;
        $items = [];
        $pieces = [];
        foreach ($package["contents"] as $item_id => $values) {
            //var_dump($values);
            //var_dump($values['line_total']);
            //get_line_total() – Calculate line total – useful for gateways.
            // $item_discounted_price = $values['line_total'] / $values['quantity'];
            //var_dump($item_discounted_price);
            //die();
            $ctr++;
            if ($values["data"]->get_meta("_exclude_flavorcloud") == "yes") {
                // $this->debug( sprintf( __( 'Skipping Product #%d is set to Exclde this product. Skipping FlavorCloud Live Rate.', 'woocommerce-shipping-flavorcloud' ), $ctr ) );
                continue;
            }
            if (!$values["data"]->needs_shipping()) {
                //  $this->debug( sprintf( __( 'Product #%d is virtual. Skipping FlavorCloud Live Rate.', 'woocommerce-shipping-flavorcloud' ), $ctr ) );
                continue;
            }
            if (!$values["data"]->get_weight()) {
                //  $this->debug( sprintf( __( 'Product #%d is missing weight. Aborting FlavorCloud Live Rate.', 'woocommerce-shipping-flavorcloud' ), $ctr ), 'error' );
                return;
            }
            //var_dump(fcsfw_convertUnits(100, 'g', 'kg'));
            // Convert the product dimensions and weight to the correct units 0.01
            //$weight = $converter->convert((float)$values['data']->get_weight())->from($weight_unit)->to(strtolower($weight_unit_api));
            $converted_weight = fcsfw_convertUnits($values["data"]->get_weight(), $this->weight_unit, strtolower($this->weight_unit_api));
            // If our product weight is too low set it to 0.01. Ex: if product is 1grams then the above will return 0KG (10grams will return 0.01)
            if ($converted_weight == 0) {
                $converted_weight = 0.01;
            }
            //var_dump($converted_weight);
            $converted_width = fcsfw_convertUnits($values["data"]->get_width(), $this->dimension_unit, strtolower($this->dimension_unit_api));
            $converted_length = fcsfw_convertUnits($values["data"]->get_length(), $this->dimension_unit, strtolower($this->dimension_unit_api));
            $converted_height = fcsfw_convertUnits($values["data"]->get_height(), $this->dimension_unit, strtolower($this->dimension_unit_api));
            // Get package weight.
            //$weight = wc_get_weight( $values['data']->get_weight(), 'lbs' );
            // Get package dimensions.
            if ($values["data"]->get_length() && $values["data"]->get_height() && $values["data"]->get_width()) {
                $item_discounted_price = $values["line_total"] / $values["quantity"];
                $items[] = ["Name" => $values["data"]->get_name(), "Width" => $converted_width, "Length" => $converted_length, "Height" => $converted_height, "Weight" => $converted_weight, "Quantity" => (float)$values["quantity"], "Price" => (float)$item_discounted_price, "SKU" => $values["data"]->get_SKU(), ];
                $pieces[] = ["Quantity" => (float)$values["quantity"], "Weight" => $converted_weight, "Length" => $converted_length, "Width" => $converted_width, "Height" => $converted_height, "SalePrice" => (float)$item_discounted_price, "HSCode" => $values["data"]->get_meta("_hscode"), "OriginCountryCode" => "US", "Material" => $values["data"]->get_meta("_material"), "Description" => $values["data"]->get_name(), "SKU" => $values["data"]->get_SKU(), "Category" => [], ];
            }
        }
        //$this->debug( fcsfw_print_array(json_encode($pieces) ) );
        //$this->debug( fcsfw_print_array($pieces) );
        $quote_box = FlavorCloud_API_BinPack($items);
        fcsfw_logger($items);
        fcsfw_logger($quote_box);
        //$this->debug( fcsfw_print_array($quote_box));
        // Build Ship to Address
        $ShipToAddress = ["Name" => WC()->customer->get_shipping_first_name() . " " . WC()->customer->get_shipping_last_name(), "AttentionName" => "", "AddressLine1" => WC()->customer->get_shipping_address(), "AddressLine2" => WC()->customer->get_shipping_address_2(), "AddressLine3" => "", "City" => WC()->customer->get_shipping_city(), "State" => WC()->customer->get_shipping_state(), "Country" => WC()->customer->get_shipping_country(), "Zip" => WC()->customer->get_shipping_postcode(), "Phone" => WC()->customer->get_billing_phone(), "Email" => WC()->customer->get_billing_email(), ];
        //$this->debug(fcsfw_print_array( $ShipToAddress));
        //die();
        $reponce = $this->Flavor_Cloud_API_GetRates($pieces, $quote_box, $ShipToAddress);
        //$this->debug(fcsfw_print_array($reponce));
        fcsfw_logger($reponce);
        //$option[$main_data['TermsOfTrade']]['LandedCostDetail']['Duty'];
        //$option[$main_data['TermsOfTrade']]['LandedCostDetail']['SalesTax'];
        //die();
        //$this->debug(fcsfw_print_array($reponce));
        //die();
        if (!empty($reponce)) {
            $main_data = FlavorCloud_API_Get_Settings("API");
            //$this->debug(fcsfw_print_array($main_data));
            // If terms of trade is set to Customs Dutied Paid add fee to cart
            // Fee #1 SalesTax
            // Fee #2 Duty
            //add_action( 'woocommerce_cart_calculate_fees','fcsfw_add_landed_cost' );
            global $woocommerce;
            // Fetch the new setting and determine whether to reverse the methods.
            $reverse_order = $main_data["fcsfw_reverse_order"];
            // If we need to reverse the order, do that here before processing them
            if ($reverse_order == "yes") {
                //$reponce = array_reverse($reponce, true);
                
            } else {
                $reponce = array_reverse($reponce, true);
            }
            // Make a shipping rate for each result
            $additional_fees = [];
            foreach ($reponce as $key => $option) {
                //$this->debug(fcsfw_print_array($reponce));
                //'TermsOfTrade' => 'DDP'
                // Make sure we are only processing the shipping rates
                if (is_array($option) && isset($option[$main_data["TermsOfTrade"]])) {
                    //$this->debug(fcsfw_print_array($reponce));
                    $additional_fees[$key] = ["Duty" => $option[$main_data["TermsOfTrade"]]["LandedCostDetail"]["Duty"], "SalesTax" => $option[$main_data["TermsOfTrade"]]["LandedCostDetail"]["SalesTax"], "AIT" => $option[$main_data["TermsOfTrade"]]["LandedCostDetail"]["AIT"], "LandedCost" => $option[$main_data["TermsOfTrade"]]["LandedCostDetail"]["LandedCost"], "DutyHashKey" => $option[$main_data["TermsOfTrade"]]["LandedCostDetail"]["DutyHashKey"], ];
                    
                    $standard_rate_override = $this->get_option("standard_shipping_rate");
                    $express_rate_override = $this->get_option("express_shipping_rate");

                    if ($key == "Standard") {
                        if (!empty($main_data["fcsfw_standard_label"])) {
                            $label = $main_data["fcsfw_standard_label"];
                        } else {
                            $label = $option[$main_data["TermsOfTrade"]]["Carrier"] . " " . $option[$main_data["TermsOfTrade"]]["Days"];
                        }
                                // Override the Standard shipping rate if the setting is set.
        if (!empty($standard_rate_override)) {
            $option[$main_data["TermsOfTrade"]]["ShippingCost"] = $standard_rate_override;
        }
                    }

                    if ($key == "Express") {
                        if (!empty($main_data["fcsfw_express_label"])) {
                            $label = $main_data["fcsfw_express_label"];
                        } else {
                            $label = $option[$main_data["TermsOfTrade"]]["Carrier"] . " " . $option[$main_data["TermsOfTrade"]]["Days"];
                        }
                                // Override the Express shipping rate if the setting is set.
        if (!empty($express_rate_override)) {
            $option[$main_data["TermsOfTrade"]]["ShippingCost"] = $express_rate_override;
        }
                    }

                    $rate = ["id" => $key . "_" . $option[$main_data["TermsOfTrade"]]["HashKey"] . "_" . $option[$main_data["TermsOfTrade"]]["LandedCostDetail"]["DutyHashKey"],
                    //'label' => $methodNames[$option->name],
                    "label" => $label, "cost" => $option[$main_data["TermsOfTrade"]]["ShippingCost"], "meta_data" => [], ];
                    if (($key == "Standard" && $main_data["fcsfw_standard_disable"] !== "yes") || ($key == "Express" && $main_data["fcsfw_express_disable"] !== "yes")) {
                        $this->add_rate($rate);
                    }
                }
            }
            // If terms of trade is set to Customs Dutied Paid add feed to cart
            // Fee #1 SalesTax
            // Fee #2 Duty
            if ($main_data["TermsOfTrade"] == "DDP") {
                // Store the additional fees in a global variable, used by lavorcloud_add_additional_fees
                $GLOBALS["flavorcloud_additional_fees"] = $additional_fees;
            }
        }
    }
}
