<?php
/**
 * Plugin Name:     FlavorCloud Shipping For Woocommerce
 * Plugin URI: https://qwebmaster.com/flavor-cloud-shipping-for-woocommerce/
 * Description:    FlavorCloud Shipping For Woocommerce is a plugin that leverages the FlavorCloud API to provide live, real-time shipping rates. It offers seamless integration with WooCommerce, accurate rate calculations based on a variety of factors such as weight, dimensions and destination, and supports both metric and imperial measurement units. A crucial part of its functionality is to virtually pack cart items for the most cost-effective shipping solution.
 * Version:         1.3.0
 * Requires at least: 5.6
 * Requires PHP:      8.0
 * Tested up to:    6.3.1
 * WC requires at least: 4.0
 * WC tested up to: 8.1.1
 * Author:          Qwebmaster
 * Author URI: https://qwebmaster.com
 * Copyright: © 2023 Qwebmaster
 * License:         GNU General Public License v3.0
 * License URI:     http://www.gnu.org/licenses/gpl-3.0.html
 * Text Domain:     flavor-cloud-shipping-for-woocommerce
 * Domain Path:     /languages
 * 
 * @package         FlavorCloud_Shipping_For_Woocommerce
 */


// Change Log: https://qwebmaster.com/flavorcloud-shipping-for-woocommerce/
/*
Version 1.3.0
* Enhanced compatibility with WooCommerce Subscriptions.
* Prevented selection of past dates in the calendar datepicker.
* Corrected typographical errors.
* Implemented various optimizations.
* Introduced a feature to set a fixed rate for each FlavorCloud shipping method, while preserving applicable taxes and fees.
*/


// DELETE , DEV ONLY!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
function display_custom_code_in_footer2() {
    //$I = new WpwebdriverTester($scenario);
}
// Hook into the wp_footer action
add_action('wp_footer', 'display_custom_code_in_footer2');
// DELETE , DEV ONLY!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!




// Define Base Plugin Dir 
define( 'FCSFW_PLUGIN_DIR_PATH', plugin_dir_path( __FILE__ ) );
define( 'FCSFW_PLUGIN_DIR_URL', plugin_dir_url( __FILE__ ) );

/** FlavorCloud_Shipping_For_Woocommerce Debug Functions */
require FCSFW_PLUGIN_DIR_PATH . 'includes/fcsfw-debug.php';

// PHP version 8.0+ Only
if (version_compare(PHP_VERSION, '8.0.0', '<')) {
    function fcsfw_incompatible_php_version_notice() {
        ?>
        <div class="notice notice-error is-dismissible">
            <p><?php _e( 'Your PHP version is ' . PHP_VERSION . ', FlavorCloud Shipping For WooCommerce requires PHP 8.0.0 or higher. Please upgrade your PHP version.', 'flavor-cloud-shipping-for-woocommerce' ); ?></p>
        </div>
        <?php
    }
    add_action( 'admin_notices', 'fcsfw_incompatible_php_version_notice' );
} else {
    // Include necessary files
    function include_wc_fcsfw_plugin_files() {
        /** FlavorCloud_Shipping_For_Woocommerce Debug Functions */
        //require FCSFW_PLUGIN_DIR_PATH . 'includes/fcsfw-debug.php';

        /** FlavorCloud_Shipping_For_Woocommerce Main WooCommerce Functions */
        require FCSFW_PLUGIN_DIR_PATH . 'includes/fcsfw-woocommerce.php';

        /** FlavorCloud_Shipping_For_Woocommerce Main API Functions */
        require FCSFW_PLUGIN_DIR_PATH . 'includes/flavorcloud/api-main.php';

        /** FlavorCloud_Shipping_For_Woocommerce admin */
        require FCSFW_PLUGIN_DIR_PATH . 'includes/flavorcloud-shipping-admin.php';

        // Uncomment the following lines to print the settings arrays
        // fcsfw_print_array(FlavorCloud_API_Get_Settings('ShipFromAddress'));
        // fcsfw_print_array(FlavorCloud_API_Get_Settings('API'));
    }
    // Run the custom function on woocommerce_init action
    add_action('woocommerce_init', 'include_wc_fcsfw_plugin_files');

    /**
     * Add Flavor Cloud Shipping Method
     */
    add_filter( 'woocommerce_shipping_methods', 'register_flavorcloud_method' );
    function register_flavorcloud_method( $methods ) {
        // Only Run On the Checkout page 
        //if ( is_cart() || is_checkout() ) {
        if ( is_cart() || is_checkout() || is_admin() || wp_doing_ajax() ) {
        require_once("includes/wc-flavorcloud-shipping-method.php");
        // $method contains available shipping methods
        $methods[ 'flavor-cloud' ] = 'WC_Shipping_FlavorCloud';
        }
        return $methods;
    }

    add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), 'add_settings_link_to_flavorcloud_plugin' );
    function add_settings_link_to_flavorcloud_plugin( $links ) {
        $settings_url = admin_url( 'admin.php?page=wc-settings&tab=shipping&section=flavor_cloud_settings' );
        $settings_link = '<a href="' . esc_url( $settings_url ) . '">' . __( 'Settings', 'flavor-cloud-shipping-for-woocommerce' ) . '</a>';
        array_unshift( $links, $settings_link );
        return $links;
    }
    
}


function adjust_shipping_rates( $rates, $package ) {
    foreach ( $rates as $rate_key => $rate ) {
        $shipping_method_id = $rate->get_method_id();
        $shipping_instance_id = $rate->get_instance_id();

        if ( 'flat_rate' === $shipping_method_id ) {
            $standard_shipping_rate = get_option( "woocommerce_flat_rate_{$shipping_instance_id}_standard_shipping_rate" );
            $express_shipping_rate = get_option( "woocommerce_flat_rate_{$shipping_instance_id}_express_shipping_rate" );

            if ( ! empty( $standard_shipping_rate ) && 'standard' === $rate->get_label() ) {
                $rates[ $rate_key ]->set_cost( $standard_shipping_rate );
            }

            if ( ! empty( $express_shipping_rate ) && 'express' === $rate->get_label() ) {
                $rates[ $rate_key ]->set_cost( $express_shipping_rate );
            }
        }
    }

    return $rates;
}
add_filter( 'woocommerce_package_rates', 'adjust_shipping_rates', 10, 2 );
